/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __PKT_TCP_HEADER_INCLUDED__
#define __PKT_TCP_HEADER_INCLUDED__

/* MISC */
#define TCP_MSS			512
#define TCP_MAXWIN		65535
#define TCP_MAX_WINSHIFT	16

/* OPTIONS */
#define TCPI_OPT_TIMESTAMPS	1
#define TCPI_OPT_SACK		2
#define TCPI_OPT_WSCALE		4
#define TCPI_OPT_ECN		8

/* TCP FLAGS */
#define TCP_FIN		0x01	/* Finish */
#define TCP_SYN		0x02	/* Synchronise */
#define TCP_RST		0x04	/* Reset */
#define TCP_PSH		0x08	/* Push */
#define TCP_ACK		0x10	/* Acknowlege */
#define TCP_URG		0x20	/* Urgent pointer */
#define TCP_ECE		0x40	/* ECN echo */
#define TCP_CWR		0x80	/* Congestion window reduced */

typedef union _tcpflags {
	u_int8_t flags;
	struct {
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t fin:1;
	u_int8_t syn:1;
	u_int8_t rst:1;
	u_int8_t psh:1;
	u_int8_t ack:1;
	u_int8_t urg:1;
	u_int8_t ece:1;
	u_int8_t cwr:1;
#elif __BYTE_ORDER == __BIG_ENDIAN
	u_int8_t cwr:1;
	u_int8_t ece:1;
	u_int8_t urg:1;
	u_int8_t ack:1;
	u_int8_t psh:1;
	u_int8_t rst:1;
	u_int8_t syn:1;
	u_int8_t fin:1;
#else
#error "Couldn't determine endianness"
#endif
	}bits;
}tcpflags;

typedef struct fs_tcphdr
{
	u_int16_t	sport,dport;
	u_int32_t	seq;
	u_int32_t	ack;
#if __BYTE_ORDER == __LITTLE_ENDIAN
	u_int8_t	res1:4;	/* ??? */
	u_int8_t	doff:4;
#elif __BYTE_ORDER == __BIG_ENDIAaN
	u_int8_t	doff:4;
	u_int8_t	res1:4;	/* ??? */
#endif
	tcpflags	flags;
	u_int16_t	win;
	u_int16_t	csum;
	u_int16_t	urp;
}fs_tcphdr_t, *p_tcp;

#endif /* __PKT_TCP_HEADER_INCLUDED__ */
