/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __TARGETS_HEADER_INCLUDED__
#define __TARGETS_HEADER_INCLUDED__
/* 
* TARGETS.H
* Manages all installed targets, also
* contains target_add(), which plugins call
* to register new targets
*/

#include <g_globals.h>
#include <packet.h>
#include <lists.h>

typedef enum _target_ret{
	TARGET_DONE,		/* Done with this packet */
	TARGET_CONTINUE		/* Carry on processing */
}target_action_e;

typedef target_action_e (*proc_target)(p_packet, void *, void *);
typedef void *(*proc_target_validate)(char *);
typedef void (*proc_target_cleanup)(void *);

typedef struct target {
	list_head_t 		l;
	char			*name;
	proc_target		target;
	proc_target_validate	validate;
	proc_target_cleanup	cleanup;
}target_t, *p_target;

/* Initialise targets subsystem  */
int target_init(void);

/* Get a target by its name */
p_target target_get(char *);

typedef int (*proc_target_add)(char *, proc_target,
	proc_target_validate, proc_target_cleanup);

#ifndef __PLUGIN__
int target_add(char *, proc_target, proc_target_validate, proc_target_cleanup);
#endif

#endif //__TARGETS_HEADER_INCLUDED__
