#include <daemon.h>
#include <fstp.h>
#include <zlib.h>

/* Global variables */
d_globals_t dglobals;

/* Main buffer */
fstp_pkt_t buf;
char *p=(char *)&buf;

/* Handy strings */
char *str_seg[]={
	"Padding",
	"Packet data",
	"Command packet",
	"Timestamp",
	"EOT"
};

char *cmp_str[]={
	"No",
	"Zlib",
	"bzip2",
};

/* Just the decompression ma'am */
u_int32_t decompress_zpacket(p_fstp_pkt p, int level)
{
	int dlen=FSTP_MAX_PAYLOAD;
	char buf[FSTP_MAX_PAYLOAD];

	/* Copy to temporary buffer */
	memcpy((char *)&buf, (char *)&p->data, p->hdr.len-sizeof(p->hdr));
	
	/* Actually uncompress */
	uncompress((char *)&p->data, &dlen, (char *)&buf, p->hdr.len-sizeof(p->hdr));

	return dlen + sizeof(p->hdr);
}

/* Decode the packet and send useful information to the
* various functions */
int decode_packet(void *pkt, u_int32_t len)
{
	void *t=pkt;
	int eot=0;
	
	if ( !pkt || !len ) return 0;

	while( (t-pkt) <= len-1 ) {
		/* TODO: Send to packet handler. Dont forget to check caplen */
		
		printf(" * %u byte segment. (Type: %s)\n",
			((p_segment_hdr)t)->caplen,
			((p_segment_hdr)t)->type < FSTP_SEG_MAX ?
			str_seg[((p_segment_hdr)t)->type] : "Unknown");

		if ( ((p_segment_hdr)t)->type == FSTP_SEG_EOT ) eot=1;
		
		t+=((p_segment_hdr)t)->caplen +
			sizeof(segment_hdr_t);
	}

	if ( !eot ) {
		printf(" * WARNING: Packet had missing EOT. Possibly mangled in transit\n");
	}
}

int parse_cmdline(int argc, char **argv)
{
	int i, ret=1;
	struct hostent *h;
	struct in_addr a;

	while( (i=getopt(argc, argv, "p:b:" )) != -1 ) {
		switch(i) {
			case 'p':
				dglobals.port=atoi(optarg);
				if ( dglobals.port == 0 ) {
					MASK_SET(dglobals.err, DERR_PORT);
					ret=0;
				}
				continue;
			case 'b':
				dglobals.addr=0xFFFFFFFF;

				if ( inet_aton(optarg, &a) ) {
					dglobals.addr=a.s_addr;
				}else if ( (h=gethostbyname(optarg)) ) {
					memcpy(&a, h->h_addr, h->h_length);
					dglobals.addr=a.s_addr;
				}
				
				if ( dglobals.addr == 0xFFFFFFFF ) {
					MASK_SET(dglobals.err, DERR_ADDR);
					ret=0;
				}
				continue;
		}
	}
	
	return ret;
}

void usage (char *cmd)
{
	printf("\nUsage: \n"
		"\t%s [OPTIONS]\n\n"
		"	-p [port]	Set which UDP port to listen on\n"
		"	-b [addr]	IP address to bind to\n"
		,cmd);

	if ( MASK(dglobals.err, DERR_PORT) )
		printf(DERR_MSG "Port cannot be %i\n", dglobals.port);
}

int main ( int argc, char **argv )
{
	int lfd=-1;
	struct sockaddr_in svr;
	struct sockaddr_in from;
	int fromlen, rlen=0;

	printf("StormWall v1.0" CRLF COPYRIGHT_STRING);

	/* Zero the globals structure and setup the
	* default values */
	memset(&dglobals, 0, sizeof(dglobals));
	dglobals.port=FSTP_DEFAULT_PORT;
	dglobals.addr=htonl(INADDR_LOOPBACK);
	tzset();
	
	/* Parse the command line arguments */
	if ( !parse_cmdline(argc, argv ) ) {
		usage(argv[0]);
		return EXIT_ERR;
	}
	

	/* Now bind to our port etc.. */
	svr.sin_family=AF_INET;
	svr.sin_port=htons(dglobals.port);
	svr.sin_addr.s_addr=dglobals.addr;
	
	if ( (lfd=socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ) {
		perror("socket() ");
		return EXIT_ERR;
	}

	if ( bind(lfd, (struct sockaddr *)&svr, sizeof(svr)) < 0 ) {
		perror("bind() ");
		close(lfd);
		return EXIT_ERR;
	}
	
	/* Print a nice message */
	printf("Stormwall listening on %s:%i\n", inet_ntoa(svr.sin_addr), dglobals.port);
	
	/* Main loop. Here we validate all input and pass it on to the 
	* decode_packet() function */
	while (1){
		memset(p, 0, sizeof(buf));
		rlen=recvfrom(lfd, p, 1024, 0, (struct sockaddr *)&from, &fromlen);
		if ( rlen < 0 ) break;
		if ( rlen == 0 ) continue;
	
		printf("<= %s:%u ", 
			inet_ntoa(from.sin_addr),
			htons(from.sin_port));

		if ( rlen != buf.hdr.len ) {
			printf("ERROR: Length mismatch %i/%u" CRLF,
				rlen, buf.hdr.len);
			continue;
		}

		if ( buf.hdr.hdr_len != sizeof(buf.hdr) ) {
			printf("ERROR: Length mismatch %u (should be: %u)" CRLF,
				 buf.hdr.hdr_len, sizeof(buf.hdr));
			continue;
		}

		printf("[len=%i] %s Compression at level %u\n",
			buf.hdr.len,
			buf.hdr.compression < FSTP_COMPRESS_MAX ? 
				cmp_str[buf.hdr.compression] : 
				"Unknown",
			buf.hdr.level);
		
		switch ( buf.hdr.compression ) {
			case FSTP_COMPRESS_ZLIB:
				buf.hdr.len=decompress_zpacket(&buf, buf.hdr.level);
				break;
			case FSTP_COMPRESS_NONE:
			default:
		}
		decode_packet(p+sizeof(buf.hdr), buf.hdr.len-sizeof(buf.hdr));
		
		printf("\n");
	}
	
	perror("recvfrom() ");
	close(lfd);
	return EXIT_OK;
}
