/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <templates.h>

int t_cmp_range(void *p1, void *p2)
{
	p_range_priv n1=(p_range_priv)p1;
	p_range_priv n2=(p_range_priv)p2;

	if ( (n1->min==n2->min) && (n1->max==n2->max) ) {
		return CMP_IDENTICAL;
	}
	
	return CMP_DIFFER;
}

int t_cmp_string(void *p1, void *p2)
{
	
	if ( strcmp((char *)p1, (char *)p2)==0 ) {
		return CMP_IDENTICAL;
	}
	
	return CMP_DIFFER;
}

int t_cmp_number(void *p1, void *p2)
{
	p_num_priv n1=(p_num_priv)p1;
	p_num_priv n2=(p_num_priv)p2;
	
	if ( n1->val == n2->val ) {
		return CMP_IDENTICAL;
	}
	
	return CMP_DIFFER;
}


void *t_validate_string(char *token, p_kv opts)
{
	return globals.fs_strdup(token);
}

void *t_validate_number(char *token, p_kv opts)
{
	p_num_priv p;
	if ( !token ) return NULL;

	if ( !(p=globals.malloc(sizeof(num_priv_t))) ) return NULL;

	p->val = atoi(token);
	return p;
}

void *t_validate_range(char *token, p_kv opts)
{
	p_range_priv priv=NULL;
	char *min=NULL, *max=NULL;
	char *tmp;
	
	if ( !token ) return NULL;

	if ( !(priv=globals.malloc(sizeof(range_priv_t))) ) {
		return NULL;
	}

	if ( !(tmp=globals.malloc(strlen(token)+1)) ) {
		globals.free(priv);
		return;
	}

	strcpy(tmp, token);
	min=tmp;

	if ( (max=strchr(tmp, ':')) ) {
		*max='\0';
		max++;
	}else{
		max=min;
	}

	priv->min=atoi(min);
	priv->max=atoi(max);
	
	globals.free(tmp);
	return (void *)priv;
}

void t_cleanup_ptr(void *priv)
{
	/* Plugins with no private data just return 0xdeadbeef */
	if ( priv && priv != 0xdeadbeef ) globals.free(priv);
}

void template_fill_struct(p_tmpl p)
{
	if ( !p ) return;

	p->t_validate_range=t_validate_range;
	p->t_validate_string=t_validate_string;
	p->t_validate_number=t_validate_number;
	
	p->t_cmp_range=t_cmp_range;
	p->t_cmp_string=t_cmp_string;
	p->t_cmp_number=t_cmp_number;
	
	p->t_cleanup_ptr=t_cleanup_ptr;
}
