/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <xmlout.h>
#include <s_util.h>

PLUGIN_DECLARE_STD_IMPORTS;

icmp_typename_t icmp_tnames[]={
	{0, "ICMP_ECHO_REPLY"},
	{3, "ICMP_DEST_UNCREACH"},
	{4, "ICMP_SOURCE_QUENCH"},
	{5, "ICMP_REDIRECT"},
	{8, "ICMP_ECHO_REQUEST"},
	{9, "ICMP_ROUTER_ADVERTISEMENT"},
	{10, "ICMP_ROUTER_SOLICITATION"},
	{11, "ICMP_TIME_EXCEED"},
	{12, "ICMP_BAD_PARAM"},
	{13, "ICMP_TIMESTAMP_REQUEST"},
	{14, "ICMP_TIMESTAMP_REPLY"},
	{15, "ICMP_INFO_REQUEST"},
	{16, "ICMP_INFO_REPLY"},
	{17, "ICMP_MASK_REQUEST"},
	{18, "ICMP_MASK_REPLY"},
};

icmp_name_t icmp_names[]={
	{3, 0, "ICMP_NET_UNREACH"},
	{3, 1, "ICMP_HOST_UNREACH"},
	{3, 2, "ICMP_PROTO_UNREACH"},
	{3, 3, "ICMP_PORT_UNCREACH"},
	{3, 4, "ICMP_FRAG_NEEDED_BUT_DF_SET"},
	{3, 5, "ICMP_SOURCE_ROUTE_FAILED"},
	{3, 6, "ICMP_NET_UNKNOWN"},
	{3, 7, "ICMP_HOST_UNKNOWN"},
	{3, 8, "ICMP_SOURCE_HOST_ISOLATED"},
	{3, 9, "ICMP_NET_ADMIN_PROHIBITED"},
	{3, 10, "ICMP_HOST_ADMIN_PROHIBITED"},
	{3, 11, "ICMP_NET_UNREACH_TOS"},
	{3, 12, "ICMP_HOST_UNREACH_TOS"},
	{3, 13, "ICMP_COMMS_FILTERED"},
	{3, 14, "ICMP_PRECEDENCE_VIOLATION"},
	{3, 15, "ICMP_PRECEDENCE_CUTOFF"},
	{5, 0, "ICMP_REDIRECT_NET"},
	{5, 1, "ICMP_REDIRECT_HOST"},
	{5, 2, "ICMP_REDIRECT_TOS_NET"},
	{5, 3, "ICMP_REDIRECT_TOS_HOST"},
	{11, 0, "ICMP_TTL_EXCEEDED"},
	{11, 1, "ICMP_TTL_EXCEEDED_IN_REASSEMBLY"},
	{12, 0, "ICMP_IP_HEADER_BAD"},
	{12, 1, "ICMP_IP_OPTION_MISSING"},
};

proc_target_add		add_target;

int afd;
mlock_t alock;

char *format_ip(uint ip)
{
	char *tmp;
	struct in_addr addr;
	addr.s_addr=ip;
	
	if (!(tmp=inet_ntoa(addr))) return NULL;

	return my_strdup(tmp);
}

const char *icmp_get_name(u_int8_t type, u_int8_t code) 
{
	int i;

	for (i=0; i<(sizeof(icmp_names)-sizeof(icmp_name_t)); i++) {
		if ( type==icmp_names[i].type && code==icmp_names[i].code ) {
			return icmp_names[i].name;
		}
	}

	for (i=0; i<(sizeof(icmp_tnames)-
		sizeof(icmp_typename_t)); i++) {
			if ( type == icmp_tnames[i].type ) {
				return icmp_tnames[i].name;
			}
	}
	
	return "ICMP_UNKNOWN";
}

void print_layer_ip(p_packet p, int i) 
{
	char *src, *dst;

	src=format_ip(p->layer[i].h.ip->saddr);
	dst=format_ip(p->layer[i].h.ip->daddr);

	file_out(afd,
		"\t\t\t\t<prop name=\"source_ip\" value=\"%s\"/>" CRLF
		"\t\t\t\t<prop name=\"dest_ip\" value=\"%s\"/>" CRLF
		"\t\t\t\t<prop name=\"ttl\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"id\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"proto\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"tos\" value=\"%u\"/>" CRLF
		,src, dst,
		p->layer[i].h.ip->ttl,
		ntohs(p->layer[i].h.ip->id),
		p->layer[i].h.ip->protocol,
		p->layer[i].h.ip->tos);

	if ( src ) my_free(src);
	if ( dst ) my_free(dst);
}

void print_layer_icmp( p_packet p, int i )
{
	/* TODO: Get type and code and work out name */
	file_out(afd, 
		"\t\t\t\t<prop name=\"name\" value=\"%s\"/>" CRLF
		"\t\t\t\t<prop name=\"type\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"code\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"csum\" value=\"0x%x\"/>" CRLF,
		icmp_get_name(p->layer[i].h.icmp->type, p->layer[i].h.icmp->code),
		p->layer[i].h.icmp->type,
		p->layer[i].h.icmp->code,
		p->layer[i].h.icmp->csum);
}

void print_layer_tcp( p_packet p, int i ) 
{
	int n,x;
	char tcpflags[]="FSRPAU12";
	char disp[9];

	file_out(afd,
		"\t\t\t\t<prop name=\"seq\" value=\"0x%x\"/>" CRLF
		"\t\t\t\t<prop name=\"ack\" value=\"0x%x\"/>" CRLF
		"\t\t\t\t<prop name=\"win\" value=\"0x%x\"/>" CRLF
		"\t\t\t\t<prop name=\"urg\" value=\"0x%x\"/>" CRLF,
		ntohl(p->layer[i].h.tcp->seq),
		ntohl(p->layer[i].h.tcp->ack),
		ntohs(p->layer[i].h.tcp->win),
		ntohs(p->layer[i].h.tcp->urp));

	for(n=0,x=1; n<8; n++,x*=2) {
		if (p->layer[i].h.tcp->flags.flags & x)  {
			disp[n]=tcpflags[n];
		}else{
			disp[n]='*';
		}
	}disp[8]='\0';
		
	file_out(afd, "\t\t\t\t<prop name=\"flags\" value=\"%s\"/>"  CRLF, (char *)&disp);
}

void print_layer_udp( p_packet p, int i )
{
	file_out(afd, 
		"\t\t\t\t<prop name=\"spt\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"dpt\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"csum\" value=\"0x%x\"/>" CRLF,
		p->layer[i].h.udp->sport,
		p->layer[i].h.udp->dport,
		p->layer[i].h.udp->csum);
}

void print_layer_gre( p_packet p, int i )
{
	file_out(afd, 
		"\t\t\t\t<prop name=\"proto\" value=\"0x%x\"/>" CRLF
		"\t\t\t\t<prop name=\"len\" value=\"%u\"/>" CRLF
		"\t\t\t\t<prop name=\"call_id\" value=\"0x%x\"/>" CRLF
		"\t\t\t\t<prop name=\"ack\" value=\"0x%x\"/>" CRLF,
		ntohs(p->layer[i].h.gre->proto),
		ntohs(p->layer[i].h.gre->len),
		ntohs(p->layer[i].h.gre->call_id),
		ntohl(p->layer[i].h.gre->u.ack));
}

void print_layer_ethernet( p_packet p, int i ) 
{
	unsigned char *x = p->layer[i].h.raw;
	int c;
	
	
	file_out(afd, "\t\t\t\t<prop name=\"mac_src\" value=\"");
	for ( c=0; c<6; c++,x++ )
		file_out(afd, "%02x%s", *x,c==5 ? "\"/>" CRLF :":");
	
	file_out(afd, "\t\t\t\t<prop name=\"mac_dst\" value=\"");
	for ( c=0; c<6; c++,x++ )
		file_out(afd, "%02X%s", *x,c==5 ? "\"/>" CRLF :":");

	file_out(afd, "\t\t\t\t<prop name=\"proto\" value=\"0x%x\"/>" CRLF,
		ntohs(p->layer[i].h.mac->proto));
}

target_action_e alert_target(p_packet p, void *priv, p_rule r)
{
	char tb[512];
	int i;

	/* Format the time */
	tb[0]='\0';

	strftime((char *)&tb, 512-1, "%a %b %d %T %Y", &p->ltime);
	
	/* Big nasty (necessary) lock */
	CRITICAL_START(alock);
	
	/* Alert message */
	file_out(afd, "\t<packet comment=\"%s\" pretty_time=\"%s\">" CRLF, priv, (char *)&tb);

	/* Layer 1, or out of band information */
	file_out(afd, "\t\t<oob len=\"%u\" caplen=\"%u\" spt=\"%u\" dpt=\"%u\"/>" CRLF,
		p->len, p->caplen,p->sport, p->dport);

	/* Information about our decode */
	file_out(afd, "\t\t<decode cap_type=\"%s\" cap_if=\"%s\">" CRLF,
		p->capdev->capture->name, p->capdev->interface);
	
	for (i=0; i<p->top_layer; i++) {
		file_out(afd, "\t\t\t<layer index=\"%i\" proto=\"%s\">" CRLF,
			i, p->layer[i].name);
	
		switch(p->layer[i].type) {
			case LTYPE_LINK:
				if ( p->layer[i].id == DLT_EN10MB ) {
					print_layer_ethernet(p, i);
				}break;
			case LTYPE_NET:
				if ( p->layer[i].id == 0x0800 ) {
					print_layer_ip(p, i);
				} break;
			case LTYPE_TRANS:
				if ( p->layer[i].id == IPPROTO_TCP ) {
					print_layer_tcp(p, i);
				}else if ( p->layer[i].id == IPPROTO_ICMP ) {
					print_layer_icmp(p, i);
				}else if ( p->layer[i].id == IPPROTO_UDP ) {
					print_layer_udp(p, i);
				}	
				break;
			case LTYPE_ENCAP:
				if ( p->layer[i].id == IPPROTO_GRE ) {
					print_layer_gre(p,i);
				}
			}

		file_out(afd, "\t\t\t</layer>" CRLF);
	}

	file_out(afd, "\t\t</decode>" CRLF "\t</packet>" CRLF CRLF);
	
	CRITICAL_STOP(alock);
	return TARGET_DONE;
}

void *alert_validate(char *token)
{
	char *tmp;
	
	if ( !token ) return NULL;

	if ( !(tmp=my_strdup((token))) ) {
		return NULL;
	}

	return tmp;
}

uint _PLUGIN_INIT (p_plugin_in in, p_plugin_out out)
{
	const char *fn;
	
	PLUGIN_CHECK_PARAMS(in);
	PLUGIN_IMPORT_STD;
	
	add_target=in->add_target;
	
	PLUGIN_NAME("target/xml");
	PLUGIN_DESC("Prints packet data to an XML file");
	PLUGIN_VERS(0,1);
	
	/* Open up out alert file */
	MUTEX_CREATE(alock, "XML target lock");
	if ( !(fn=get_var("XML_FILE")) ) {
		fn="xml.log";
	}

	if ( (afd=open(fn, O_WRONLY | O_TRUNC | O_CREAT, 00600)) < 0 ) {
		print_out("XML target unable to open logfile (%s)" CRLF, fn);
		return PLUGIN_ERR_FAIL;
	}

	file_out(afd, "<?xml version=\"1.0\"?>" CRLF
		"<firestorm version=\"0.1\">" CRLF CRLF);
	
	add_target("XML",
		alert_target,
		alert_validate,
		in->templates.t_cleanup_ptr);
	
	return PLUGIN_ERR_OK;
}

/* This is where we would cleanup our shit */
uint _PLUGIN_UNLOAD (void)
{
	file_out(afd, "</firestorm>" CRLF);
	close(afd);
	MUTEX_DESTROY(alock);
	return PLUGIN_ERR_OK;
}
